@@ -81,12 +81,14 @@ static SelectorData *selector_data;	/* data about the selector */
 
 
 /* Forward declarations of internal functions */
+static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
+				    gpointer user_data);
 void board_gtk_render (void);
 static void render_tile (Tile *tile, gint row, gint col);
 GtkWidget* create_tile (double x, double y, Tile *tile);
 void move_item (GtkWidget *item, ItemDirection direc);
 int move_item_anim (void *data);
-static GtkWidget *get_item_by_row_col (gint row, gint col);
+static GtkWidget *get_item_by_row_col (guint row, guint col);
 
 static void selector_move_to (SelectorData *data, guint row, guint col);
 static void selector_unselect (SelectorData *data);
@@ -99,7 +101,7 @@ static void selector_arrows_hide (SelectorData *data);
 
 /* Function implementations */
 
-static GtkWidget *get_item_by_row_col (gint row, gint col)
+static GtkWidget *get_item_by_row_col (guint row, guint col)
 {
   gint width, height;
   gint item_point_x, item_point_y;
@@ -167,10 +169,12 @@ void move_item (GtkWidget *item, ItemDirection direc)
       case RIGHT:
         tmp_col = tmp_col + 1;
         break;
+      default:
+	break;
     }
 
-    if (tmp_row < 0 || tmp_row >= playfield_get_n_rows (board_sce) ||
-        tmp_col < 0 || tmp_col >= playfield_get_n_cols (board_sce))
+    if (tmp_row >= playfield_get_n_rows (board_sce) ||
+        tmp_col >= playfield_get_n_cols (board_sce))
       break;
 
     tile = playfield_get_tile (board_sce, tmp_row, tmp_col);
@@ -220,7 +224,7 @@ void move_item (GtkWidget *item, ItemDirection direc)
 }
 
 int move_item_anim (void *data) {
-  AnimData *anim_data = (AnimData *) data;
+  //AnimData *anim_data = (AnimData *) data;
   gint x, y;
 
   if (anim_data->counter > 0) {
@@ -248,6 +252,7 @@ static gboolean board_handle_arrow_event (GtkWidget *item,
                                           GdkEventButton *event,
                                           gpointer direction)
 {
+  gtk_widget_grab_focus (GTK_WIDGET (board_canvas));
   /* is currently an object moved? */
   if (anim_data->timeout_id != -1)
     return FALSE;
@@ -429,16 +434,17 @@ void board_gtk_init (Theme * theme, gpointer canvas)
 
   create_background_floor ();
   create_logo ();
-  gdk_window_set_background_rgba (GDK_WINDOW (gtk_widget_get_window (
-                                    GTK_WIDGET (gtk_widget_get_parent (canvas)))),
-                                  theme_get_background_color (theme));
   gtk_widget_show_all (GTK_WIDGET(board_canvas));
 
+    /* add playfield canvas to left side */
+  g_signal_connect (GTK_WIDGET(board_canvas), "key-press-event",
+		    G_CALLBACK (on_key_press_event), app);
+
   selector_data = selector_create ();
 }
 
-void board_gtk_render () {
-  gint row, col;
+void board_gtk_render (void) {
+  guint row, col;
   Tile *tile;
 
   g_return_if_fail (board_theme != NULL);
@@ -477,35 +483,25 @@ void board_gtk_render () {
 }
 
 static void render_tile (Tile *tile, gint row, gint col) {
-  gboolean create = FALSE;
   TileType type;
   gint x, y;
 
   type = tile_get_tile_type (tile);
   switch (type) {
     case TILE_TYPE_ATOM:
-      create = TRUE;
-      break;
-
     case TILE_TYPE_WALL:
-      create = TRUE;
-      break;
-
     case TILE_TYPE_SHADOW:
-      create = TRUE;
+      convert_to_canvas (board_theme, board_env, row, col, &x, &y);
+      create_tile (x, y, tile);
       break;
 
     case TILE_TYPE_UNKNOWN:
     case TILE_TYPE_FLOOR:
+    case TILE_TYPE_NONE:
+    case TILE_TYPE_LAST:
     default:
       break;
   }
-
-
-  if (create) {
-    convert_to_canvas (board_theme, board_env, row, col, &x, &y);
-    create_tile (x, y, tile);
-  }
 }
 
 static void show_sensitive (GtkWidget *widget)
@@ -527,6 +523,7 @@ static gboolean board_handle_item_event (GtkWidget *item,
   gboolean just_unselect;
   guint new_row, new_col;
 
+  gtk_widget_grab_focus (GTK_WIDGET (board_canvas));
   /* is currently an object moved? */
   if (anim_data->timeout_id != -1)
     return FALSE;
@@ -764,7 +761,7 @@ gboolean board_gtk_handle_key_event (GObject * canvas, GdkEventKey * event,
                                      gpointer data)
 {
   GtkWidget *item;
-  gint new_row, new_col;
+  guint new_row, new_col;
   Tile *tile;
 
   g_return_val_if_fail (selector_data != NULL, FALSE);
@@ -805,7 +802,7 @@ gboolean board_gtk_handle_key_event (GObject * canvas, GdkEventKey * event,
       selector_data->mouse_steering = FALSE;
       if (!selector_data->selected) {
         new_col--;
-        if (new_col >= 0) {
+        if (new_col < playfield_get_n_cols (board_env)) {
           selector_show (selector_data);
           selector_move_to (selector_data, new_row, new_col);
         }
@@ -833,7 +830,7 @@ gboolean board_gtk_handle_key_event (GObject * canvas, GdkEventKey * event,
       selector_data->mouse_steering = FALSE;
       if (!selector_data->selected) {
         new_row--;
-        if (new_row >= 0) {
+        if (new_row < playfield_get_n_rows (board_env)) {
           selector_show (selector_data);
           selector_move_to (selector_data, new_row, new_col);
         }
@@ -862,6 +859,14 @@ gboolean board_gtk_handle_key_event (GObject * canvas, GdkEventKey * event,
   return FALSE;
 }
 
+static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
+				    gpointer user_data)
+{
+  if ((app->state == GAME_STATE_RUNNING) || (app->state == GAME_STATE_RUNNING_UNMOVED))
+    return board_gtk_handle_key_event (NULL, event, NULL);
+
+  return FALSE;
+}
 
 static void selector_move_to (SelectorData *data, guint row, guint col)
 {
@@ -906,7 +911,7 @@ static void check_for_arrow (gint r, gint c, GtkWidget *arrow) {
 
 static void selector_arrows_show (SelectorData *data)
 {
-  gint r, c;
+  guint r, c;
 
   if (board_sce == NULL)
     {
@@ -916,7 +921,7 @@ static void selector_arrows_show (SelectorData *data)
 
   r = data->row - 1;
   c = data->col;
-  if (r >= 0)
+//  if (r >= 0)
     check_for_arrow (r, c, data->arrow_top);
 
   r = data->row;
@@ -934,7 +939,7 @@ static void selector_arrows_show (SelectorData *data)
   r = data->row;
   c = data->col - 1;
 
-  if (c >= 0)
+//  if (c >= 0)
     check_for_arrow (r, c, data->arrow_left);
 
   if (data->mouse_steering)
@@ -950,9 +955,9 @@ static void selector_arrows_show (SelectorData *data)
 
 static void selector_select (SelectorData *data, GtkWidget *item)
 {
-  g_return_if_fail (data != NULL);
   gint x, y;
 
+  g_return_if_fail (data != NULL);
   gtk_container_child_get (GTK_CONTAINER(board_canvas), item, "x", &x, "y", &y, NULL);
 
   data->selected = TRUE;
