@@ -49,8 +49,6 @@ static gboolean set_next_level (void);
 static void setup_level (void);
 static void level_cleanup_view (void);
 static void atomix_exit (void);
-static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
-				    gpointer user_data);
 static void game_init (void);
 static void update_statistics (void);
 static void view_congratulations (void);
@@ -232,6 +230,7 @@ static void controller_handle_action (GameAction action)
 
   update_menu_item_state ();
   update_statistics ();
+  gtk_widget_grab_focus (GTK_WIDGET (app->fi_matrix));
 }
 
 static void
@@ -374,15 +373,6 @@ static void atomix_exit (void)
 
 }
 
-static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
-				    gpointer user_data)
-{
-  if ((app->state == GAME_STATE_RUNNING) || (app->state == GAME_STATE_RUNNING_UNMOVED))
-    return board_gtk_handle_key_event (NULL, event, NULL);
-
-  return FALSE;
-}
-
 static void game_init (void)
 {
   g_return_if_fail (app != NULL);
@@ -633,10 +623,6 @@ static AtomixApp *create_gui (GApplication *app_instance)
   gtk_grid_attach_next_to (GTK_GRID (stats_grid), app->clock,
                            time_label, GTK_POS_RIGHT, 1, 1);
 
-  /* add playfield canvas to left side */
-  g_signal_connect (G_OBJECT (app->mainwin), "key-press-event",
-		    G_CALLBACK (on_key_press_event), app);
-
   app->lb_level = GTK_WIDGET (gtk_builder_get_object (builder, "level_value"));
   app->lb_name = GTK_WIDGET (gtk_builder_get_object (builder, "molecule_value"));
   app->lb_formula = GTK_WIDGET (gtk_builder_get_object (builder, "formula_value"));
@@ -646,6 +632,16 @@ static AtomixApp *create_gui (GApplication *app_instance)
 
   headerbar = GTK_WIDGET (gtk_builder_get_object(builder, "headerbar"));
   gtk_application_add_window (GTK_APPLICATION (app->app_instance), GTK_WINDOW (app->mainwin));
+
+  const char *new_game_accels[] = {"<Primary>N", NULL};
+  gtk_application_set_accels_for_action (GTK_APPLICATION (app->app_instance), "win.GameNew", new_game_accels);
+  const char *pause_game_accels[] = {"<Primary>P", NULL};
+  gtk_application_set_accels_for_action (GTK_APPLICATION (app->app_instance), "win.GamePause", pause_game_accels);
+  const char *skip_level_accels[] = {"<Primary>S", NULL};
+  gtk_application_set_accels_for_action (GTK_APPLICATION (app->app_instance), "win.LevelSkip", skip_level_accels);
+  const char *undo_move_accels[] = {"<Primary>Z", NULL};
+  gtk_application_set_accels_for_action (GTK_APPLICATION (app->app_instance), "win.GameUndo", undo_move_accels);
+
   gtk_window_set_titlebar (GTK_WINDOW (app->mainwin), headerbar);
   GMenu * menu = G_MENU (gtk_builder_get_object (builder, "primary-menu"));
 
